package de.ullisroboterseite.ursai2actionbar;

import com.google.appinventor.components.annotations.*;
import com.google.appinventor.components.annotations.androidmanifest.*;
import com.google.appinventor.components.common.*;
import com.google.appinventor.components.runtime.*;
import com.google.appinventor.components.runtime.util.*;

import android.util.Log;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TableLayout;
import android.widget.TextView;

import java.util.*;

public class ActionBar implements View.OnClickListener {
    static final String LOG_TAG = UrsAI2ActionBar.LOG_TAG;

    final Form form;
    final UrsAI2ActionBar parent;

    LinearLayout linearLayout;
    int backgroundColor = 0xFF4862A3;
    int sizeIcon = 25;
    int pixels = 0;
    Typeface materialFont;

    TextView tvTitle;
    String title = "Screen";
    boolean fontBold = true;
    float fontSize = 18.0f;
    int colorFont = 0xFFFFFFFF; // Component.DEFAULT_VALUE_COLOR_WHITE;
    int textAlignment = 0;

    int menuFontSize = 16;
    YailList menuItemList = new YailList();
    int menuBackgroundColor = 0;
    int menuTextColor = 0; // Default

    TextView buttonLeft;
    String buttonLeftSymbol = "menu";
    int buttonLeftTextColor = Component.COLOR_DEFAULT;
    String leftButtonImagePath = "";
    String rightButtonImagePath = "";

    TextView buttonRight;
    String buttonRightSymbol = "more_vert";
    int buttonRightTextColor = Component.COLOR_DEFAULT;

    TextView button1;
    String button1Symbol = "";
    int button1TextColor = Component.COLOR_DEFAULT;

    TextView button2;
    String button2Symbol = "";
    int button2TextColor = Component.COLOR_DEFAULT;

    public ActionBar(UrsAI2ActionBar parent, Form form, Typeface materialFont) {
        this.parent = parent;
        this.form = form;
        this.materialFont = materialFont;

        // Views erzeugen und konfigurieren
        float density = form.getResources().getDisplayMetrics().density;
        pixels = (int) (((float) 56) * density);
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, pixels);

        linearLayout = new LinearLayout(form);
        linearLayout.setOrientation(LinearLayout.HORIZONTAL);
        linearLayout.setLayoutParams(params);
        linearLayout.setGravity(Gravity.CENTER_HORIZONTAL);
        linearLayout.setBackgroundColor(backgroundColor);

        int i = (int) (((float) 12) * density);

        buttonLeft = new TextView(form);
        buttonLeft.setTypeface(materialFont);
        buttonLeft.setTextSize((float) sizeIcon);
        buttonLeft.setTextColor(buttonLeftTextColor);
        if (buttonLeftTextColor == Component.COLOR_DEFAULT)
            buttonLeft.setTextColor(colorFont);
        else
            buttonLeft.setTextColor(buttonLeftTextColor);
        buttonLeft.setGravity(Gravity.CENTER);
        buttonLeft.setPadding(0, 0, 0, 0);
        buttonLeft.setLayoutParams(new LinearLayout.LayoutParams(pixels, pixels));
        setButtonCode(buttonLeft, buttonLeftSymbol);
        buttonLeft.setText(buttonLeftSymbol);
        buttonLeft.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                onLeftButtonClick();
            }
        });

        setImage(buttonLeft, leftButtonImagePath);

        linearLayout.addView(buttonLeft);

        tvTitle = new TextView(this.form);
        tvTitle.setText(this.title);

        if (fontBold)
            tvTitle.setTypeface(null, Typeface.BOLD);
        else
            tvTitle.setTypeface(null, Typeface.NORMAL);

        tvTitle.setTextSize((float) fontSize);
        tvTitle.setTextColor(colorFont);
        tvTitle.setGravity(Gravity.CENTER_HORIZONTAL + Gravity.CENTER_VERTICAL + 2); // 19 = 0x13
        tvTitle.setPadding(0, 0, 0, 0);

        // Funktioniert nur mit TableLayout.LayoutParams
        tvTitle.setLayoutParams(new TableLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.MATCH_PARENT, 1.0f));

        tvTitle.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                onLeftButtonClick();
            }
        });

        setImage(buttonRight, leftButtonImagePath);

        linearLayout.addView(tvTitle);

        button2 = new TextView(form);
        button2.setTypeface(materialFont);
        button2.setTextSize((float) sizeIcon);
        if (button2TextColor == Component.COLOR_DEFAULT)
            button2.setTextColor(colorFont);
        else
            button2.setTextColor(button2TextColor);
        button2.setLayoutParams(new LinearLayout.LayoutParams(pixels, pixels));
        button2.setGravity(Gravity.CENTER); // 17
        setButtonCode(button2, button2Symbol);
        button2.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                onButton2Click();
            }
        });
        linearLayout.addView(button2);

        button1 = new TextView(form);
        button1.setTypeface(materialFont);
        button1.setTextSize((float) sizeIcon);
        if (button1TextColor == Component.COLOR_DEFAULT)
            button1.setTextColor(colorFont);
        else
            button1.setTextColor(button1TextColor);
        button1.setLayoutParams(new LinearLayout.LayoutParams(pixels, pixels));
        button1.setGravity(Gravity.CENTER); // 17
        setButtonCode(button1, button1Symbol);
        button1.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                onButton1Click();
            }
        });
        linearLayout.addView(button1);

        buttonRight = new TextView(form);
        buttonRight.setTypeface(materialFont);
        buttonRight.setTextSize((float) sizeIcon);
        if (buttonRightTextColor == Component.COLOR_DEFAULT)
            buttonRight.setTextColor(colorFont);
        else
            buttonRight.setTextColor(buttonRightTextColor);
        buttonRight.setLayoutParams(new LinearLayout.LayoutParams(pixels, pixels));
        buttonRight.setGravity(Gravity.CENTER); // 17
        setButtonCode(buttonRight, buttonRightSymbol);
        buttonRight.setOnClickListener(this);
        linearLayout.addView(buttonRight);
    }

    /**
     * Bindet die Actionbar in das HorizontalArrangement ein
     * @param placeholder
     */
    void InitActionBar(HorizontalArrangement placeholder) {
        if (placeholder == null) {
            return;
        }

        LinearLayout.LayoutParams tpr = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT,
                ViewGroup.LayoutParams.MATCH_PARENT);
        placeholder.Height(56);
        placeholder.Width(Component.LENGTH_FILL_PARENT);
        ViewGroup horizontalarra = (ViewGroup) placeholder.getView();
        horizontalarra.removeAllViews();
        horizontalarra.addView(linearLayout, tpr);

        Log.d(LOG_TAG, "BL width: " + buttonLeft.getWidth());
        Log.d(LOG_TAG, "BL height: " + buttonLeft.getHeight());
    }

    /**
     * Handler für rechten Button, erzeugt ein Popup-Menü
     * @param v Wird nicht benutzt
     */
    public void onClick(View v) {
        parent.RightButtonClick();
    }

    /**
     * Ändert das Buttonsymbol.
     * @param button Der anzupassende Button.
     * @param symbol Das anzuzeigende Symbol (Unicode des Fonts), "" für ausschalten., "blank" für leere Anzeige
     */
    private void setButtonCode(TextView button, String symbol) {
        if (!symbol.isEmpty()) {
            if (symbol.trim().toLowerCase().equals("blank"))
                button.setText(" ");
            else
                button.setText(symbol);
            button.setVisibility(View.VISIBLE);
        } else {
            button.setText("");
            button.setVisibility(View.GONE);
        }
    }

    /**
     * Ändert das Button-Hintergrundbild.
     * @param button Der anzupassende Button.
     * @param symbol Der Asset-Pfad für das Bild
     */
    private void setImage(TextView button, String ImagePath) {
        try {
            if (!ImagePath.isEmpty()) {
                Drawable drawable = Drawable.createFromStream(form.openAsset(ImagePath), null);
                button.setBackground(drawable);
            } else
                button.setBackground(null);

        } catch (Exception e) {
            Log.d(LOG_TAG, e.toString());
        }
    }

    // ====================================================
    // Handler
    // ====================================================

    /**
     * Click-Hadler für buttonLeft
     */
    public void onLeftButtonClick() {
        parent.LeftButtonClick();
    }

    /**
    * Click-Hadler für button1
    */
    public void onButton1Click() {
        parent.Button1Click();
    }

    /**
    * Click-Hadler für button2
    */
    public void onButton2Click() {
        parent.Button2Click();
    }

    // ====================================================
    // Hintergrund
    // ====================================================

    /**
    * Legt die Hintergrundfarbe für die ActionBar fest.
    * @param argb Die zu verwendende Farbe.
    */
    public void setBackgroundColor(int argb) {
        backgroundColor = argb;
        linearLayout.setBackgroundColor(backgroundColor);
    }

    /**
     * Ruft die aktuelle Hintergrundfarbe der ActionBar ab.
     * @return Die aktuelle Hintergrundfarbe.
     */
    public int getBackgroundColor() {
        return backgroundColor;
    }

    // ====================================================
    // Titel
    // ====================================================

    /**
     * Legt den Titeltext fest.
     * @param value Der Text für den Titel
     */
    public void setTitle(String value) {
        this.title = value;
        TextViewUtil.setText(tvTitle, value);
    }

    /**
     * Ruft den Titeltext ab.
     * @return Der Titeltext.
     */
    public String getTitle() {
        return title;
    }

    /**
     * Legt die Schriftgröße fest.
     * @param size Die zu benutzende Schriftgröße.
     */
    public void setFontSize(float size) {
        fontSize = size;
        TextViewUtil.setFontSize(tvTitle, size);
    }

    /**
     * Ruft die Schriftgröße ab.
     * @return Die aktuelle Schriftgröße in DIP.
     */
    public float getFontSize() {
        return fontSize;
    }

    /**
     * Legt fest ob der Titeltext fett angezeigt werden soll.
     * @param value true oder false.
     */
    public void setFontBold(boolean value) {
        fontBold = value;

        if (fontBold)
            tvTitle.setTypeface(null, Typeface.BOLD);
        else
            tvTitle.setTypeface(null, Typeface.NORMAL);
        tvTitle.requestLayout();
    }

    /**
     * Ruft ab, ob der Titeltext fett dargestellt werden soll.
     * @return
     */
    public boolean getFontBold() {
        return fontBold;
    }

    /**
     * Legt die Textfarbe fest.
     * @param argb Die zu verwendende Textfarbe.
     */
    public void setTextColor(int argb) {
        colorFont = argb;

        TextViewUtil.setTextColor(tvTitle, argb);
        TextViewUtil.setTextColor(buttonRight, argb);
        TextViewUtil.setTextColor(buttonLeft, argb);
    }

    /**
     * Ruft die aktuelle Textfarbe ab.
     * @return Die aktuelle Textfarbe.
     */
    public int getTextColor() {
        return colorFont;
    }

    /**
     * Ruft die Ausrichtung des Titeltextes ab.
     * @return Die Ausrichtung des Titeltextes.
     */
    public int getTextAlignment() {
        return textAlignment;
    }

    /**
     * Legt die Ausrichtung des Titeltextes fest.
     * @param alignment Die Ausrichtung des Titeltextes.
     */
    public void setTextAlignment(int alignment) {
        textAlignment = alignment;
        TextViewUtil.setAlignment(tvTitle, alignment, true);
    }

    // ====================================================
    // Left Button
    // ====================================================

    /**
     * Legt das Symbol für die linke Schaltfläche fest.
     * @param name Das Symbol für die linke Schaltfläche.
     */
    void setLeftButtonSymbol(String value) {
        buttonLeftSymbol = value.toLowerCase().trim();
        setButtonCode(buttonLeft, buttonLeftSymbol);
    }

    /**
     * Ruft die Textfarbe der linken Schaltfläche ab.
     * @return Textfarbe der linken Schaltfläche, 0 (Default) für Titeltextfarbe
     */
    public int getLeftButtonTextColor() {
        return buttonLeftTextColor;
    }

    /**
     * Legt die Textfarbe für die linke Schaltfläche fest
     * @param argb Textfarbe für die linke Schaltfläche, 0 (Default) für Titeltextfarbe
     */
    public void setLeftButtonTextColor(int argb) {
        buttonLeftTextColor = argb;
        if (buttonLeftTextColor == Component.COLOR_DEFAULT)
            buttonLeft.setTextColor(colorFont);
        else
            buttonLeft.setTextColor(buttonLeftTextColor);
    }

    /**
    * Legt das Hintergrundbild für die linke Schaltfläche fest
    * @param value Asset-Pfad für das Bild
    */
    public void setLeftButtonImage(String value) {
        leftButtonImagePath = value;
        setImage(buttonLeft, leftButtonImagePath);
    }

    /**
    * Ruft den Asset-Pfad für das Hintergrundbild der linken Schaltfläche ab.
    * @return Asset-Pfad für das Hintergrundbild der linken Schaltfläche.
    */
    public String getLeftButtonImage() {
        return leftButtonImagePath;
    }

    // ====================================================
    // Right Button
    // ====================================================

    /**
     * Legt das Symbol für die rechte Schaltfläche fest.
     * @param name Das Symbol für die rechte Schaltfläche.
     */
    public void setRightButtonSymbol(String name) {
        buttonRightSymbol = name.toLowerCase().trim();
        setButtonCode(buttonRight, buttonRightSymbol);
    }

    /**
     * Legt die Textfarbe die rechte Schaltfläche fest
     * @param argb Textfarbe für die rechte Schaltfläche, 0 (Default) für Titeltextfarbe
     */
    public void setRightButtonTextColor(int argb) {
        buttonRightTextColor = argb;
        if (buttonRightTextColor == Component.COLOR_DEFAULT)
            buttonRight.setTextColor(colorFont);
        else
            buttonRight.setTextColor(buttonRightTextColor);
    }

    /**
     * Ruft die Textfarbe der rechten Schaltfläche ab.
     * @return Textfarbe der rechten Schaltfläche, 0 (Default) für Titeltextfarbe
     */
    public int getRightButtonTextColor() {
        return buttonRightTextColor;
    }

        /**
    * Legt das Hintergrundbild für die rechte Schaltfläche fest
    * @param value Asset-Pfad für das Bild
    */
    public void setRightButtonImage(String value) {
        rightButtonImagePath = value;
        setImage(buttonRight, rightButtonImagePath);
    }

    /**
    * Ruft den Asset-Pfad für das Hintergrundbild der rechten Schaltfläche ab.
    * @return Asset-Pfad für das Hintergrundbild der rechten Schaltfläche.
    */
    public String getRightButtonImage() {
        return rightButtonImagePath;
    }
    // ====================================================
    // Button #1
    // ====================================================

    /**
     * Legt das Symbol für Button #1 fest.
     * @param name Das Symbol für Button #1 .
     */
    public void setButton1Symbol(String name) {
        button1Symbol = name;
        setButtonCode(button1, button1Symbol);
    }

    /**
     * Legt die Textfarbe für Button #1 fest
     * @param argb Textfarbe für Button #1, 0 (Default) für Titeltextfarbe
     */
    public void setButton1TextColor(int argb) {
        button1TextColor = argb;
        if (button1TextColor == Component.COLOR_DEFAULT)
            button1.setTextColor(colorFont);
        else
            button1.setTextColor(button1TextColor);
    }

    /**
     * Ruft die Textfarbe von Button #1 ab.
     * @return Textfarbe von Button #1, 0 (Default) für Titeltextfarbe
     */
    public int getButton1TextColor() {
        return button1TextColor;
    }

    // ====================================================
    // Button #2
    // ====================================================

    /**
     * Legt das Symbol für Button #2 fest.
     * @param name Das Symbol Button #2.
     */
    public void setButton2Symbol(String name) {
        button2Symbol = name;
        setButtonCode(button2, button2Symbol);
    }

    /**
     * Legt die Textfarbe für Button #2 fest
     * @param argb Textfarbe für Button #2, 0 (Default) für Titeltextfarbe
     */
    public void setButton2TextColor(int argb) {
        button2TextColor = argb;
        if (button2TextColor == Component.COLOR_DEFAULT)
            button2.setTextColor(colorFont);
        else
            button2.setTextColor(button2TextColor);
    }

    /**
     * Ruft die Textfarbe von Button #2 ab.
     * @return Textfarbe von Button #2, 0 (Default) für Titeltextfarbe
     */
    public int getButton2TextColor() {
        return button2TextColor;
    }

}